`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// LIP_4digit : LOGSYS IP knyvtri elem kijelzvezrlsre
// Ngy szmjegyes kijelz modul hexadecimlis vagy BCD rtkek kijelzsre
// Az adatbemenet 16 prhuzamos vonal a 4 szmjegy bitjei szmra s egy kln 
// 4 bites busz a tizedes (hexadecimlis) pontok vezrlsre. A bemeneti 
// rtk lehet hexadecimlis vagy BCD kdban, minden kd rvnyes szegmenskpet 
// eredmnyez. A kijelzs idmultiplex mdon trtnik, 1kHz krli kapcsolsi 
// frekvencival (a 16MHz rajelbl szrmaztatva), ami szp megjelentst eredmnyez. 
// Megjegyzs: Az egysg mkdsi mdjbl kvetkezen az adatbemeneti vonalak
// rtknek vltoztatsa a kijelzsi llapothoz kpest tetszleges idpontban
// megtrtnhet, az adatbemenet rajelhez szinkronizcijra nincs szksg.
//////////////////////////////////////////////////////////////////////////////////
module LIP_4digit(
            input  clk16M,                   	// 16MHz frekvencij rajel
            input  rst,                         // Szinkron RESET jel
            input  [15:0] number,               // 16 bites szm bemenet
            input  [ 3:0] dp,                   // Tizedes pontok
            output [ 7:0] seg,                  // Szegmens kimenetek
            output reg [ 3:0] dig               // Digit kimenetek
                 );


///////////////////////////////////////////////////////////////////////////////////
// A lptets frekvencija megkzelten 1kHz, amit a 16MHz-es rajelbl lltunk el
// A binris szmllval alap oszt minden 2^14 rajelben elllt egy lptet jelet
///////////////////////////////////////////////////////////////////////////////////

reg [13:0] div;                                 // rajeloszt szmll
wire rate = &div;	                              // Vgrtk pulzus kiadsa, 
                                                // ha minden bit magas


always @ (posedge clk16M)                       // A szmll a 16MHz-rl fut	
   if (rst)  div <= 14'b0;                      // Kezdllapot belltsa
	else      div <= div + 1;                    // Szmlls felfel
 
//////////////////////////////////////////////////////////////////////////////////
// A 4 szmjegyes idmultiplex vezrlst egy ngybites gyrs szmll vezrli,
// a szmjegyek megjelentse sorrendje balrl jobbra, MSD -> LSD irnyba trtnik
// A szmll engedlyezhet, lptetse a RATE jel temben trtnik
// A dig bitekkel kzvetlenl vlasztunk a bemeneti 16 bites adat szmjegyei kzl
// s egyttal a megfelele tizedes pontot is kivlasztjuk (ha hasznljuk)
///////////////////////////////////////////////////////////////////////////////////

always	 @ (posedge clk16M)								// 16MHz-rl, a RATE jel temben		
   if (rst)       dig <= 4'b1000;                  // Kezdllapot, digit[3] aktv
	else if (rate) dig <= {dig[0], dig[3:1]};       // majd a rate jel temben lp
                                                            
wire [3:0] value;
wire  decp;

assign value = ({4{dig[3]}} & number[15:12]) | 		// Elosztott multiplexer hlzat, 
					({4{dig[2]}} & number[11: 8]) | 		// a hatkony erforrs kihasznlshoz
					({4{dig[1]}} & number[ 7: 4]) |     // Egy 4 bites 4:1-be busz MUX-ot valst 
					({4{dig[0]}} & number[ 3: 0]);      // meg, AND-OR hlzattal

assign decp =  (dig[3] & dp[3]) |						// Ugyanez 1 biten a 4 tizedes pontra
               (dig[2] & dp[2]) |
 					(dig[1] & dp[1]) |
					(dig[0] & dp[0]);
///////////////////////////////////////////////////////////////////////////////////
// A kivlasztott szmrtket 7 szegmenses kijelz kdd konvertljuk
// Az tkdol logikt pozitv logika szerint rjuk fel, azaz az aktv szegmensek rtke 1
// A szegmenskp kdolsa a kvetkez:
//      0            
//     ---
//  5 |   | 1
//     --- <--6
//  4 |   | 2
//     ---  .
//      3   7 A tizedespont a 7. bit, ezt a dekodols utn illesztjk be
///////////////////////////////////////////////////////////////////////////////////

reg [6:0] segment;											// Vltoz deklarci

always @(*)                   		           		// Kdkonverter kombincis hlzat,  
   begin                                           // Lnyegben egy 16*7 bites ROM
      case (value)            //6543210  szegmensek   
         4'b0000 : segment = 7'b0111111;   // 0
         4'b0001 : segment = 7'b0000110;   // 1
         4'b0010 : segment = 7'b1011011;   // 2
         4'b0011 : segment = 7'b1001111;   // 3
         4'b0100 : segment = 7'b1100110;   // 4
         4'b0101 : segment = 7'b1101101;   // 5
         4'b0110 : segment = 7'b1111101;   // 6
         4'b0111 : segment = 7'b0000111;   // 7
         4'b1000 : segment = 7'b1111111;   // 8
         4'b1001 : segment = 7'b1101111;   // 9
         4'b1010 : segment = 7'b1110111;   // A
         4'b1011 : segment = 7'b1111100;   // b
         4'b1100 : segment = 7'b0111001;   // C
         4'b1101 : segment = 7'b1011110;   // d
         4'b1110 : segment = 7'b1111001;   // E
         4'b1111 : segment = 7'b1110001;   // F
         default : segment = 7'b0000000;   // 0
      endcase
   end 
 
assign seg = {decp,segment};       // Tizedes pont s szegmenskd

endmodule 


